//
//	Includes
//
#include	"ColdFire.h"

//
//	Prototypes
//
void InitI2c(UINT32 clock);
__declspec(interrupt) void I2cHandler(void);
UINT32 I2cSend(UINT8 *data);
UINT32 I2cGet(UINT8 *data);
UINT32 I2cRepeatGet(UINT8 *data);
UINT8 GetChar();
UINT8 GetHexChar();
UINT8 MakeHexDigit(UINT8 data);
UINT16 MakeHex2(UINT8 data);
UINT16 GetHexByte();
UINT32 I2cParser(UINT8 *ptr);
void SystemSleep(void);
UINT8 *MemoryAlloc(UINT16 size);
void MemoryFree(UINT8 *ptr);
void BufCopy(void *dst, void *src, UINT32 len);
void TermPutc(UINT8 ch);

//
//	Constants
//

//
//	Vars
//
UINT8	*I2cData;			// Send & receive data buffer
UINT8	*p;						// Parse pointer
UINT8	I2cIndex;			// Data store & read index
UINT8	I2cSize;			// Data transfer size
UINT8	I2cRepeatAdrs;// Repeat condition adrs
UINT8	I2cRepeatSize;// Data transfer size
UINT8	I2cRecvMode;	// Recv mode flag
UINT8	I2cRepeat;		// Repeat condition start flag		
UINT8	I2cResult;		// I2c result flag

//
//	Init I2C controler
//
void InitI2c(UINT32 clock)
{
	MCF_GPIO_PASPAR = 0					// Set Port AS for I2C SCL & SDA purpose
		| MCF_GPIO_PASPAR_SCL_SCL	// Use PAS0 as a I2C SCL
		| MCF_GPIO_PASPAR_SDA_SDA;// Use PAS1 as a I2C SDA

	MCF_I2C_I2FDR = clock;			// I2C rate set
	MCF_I2C_I2ADR = 0x00;				// I2C slave adrs set (no use)

	MCF_I2C_I2CR = 0				// Set I2C control register
		| MCF_I2C_I2CR_IIEN		// Enable Interrupts
		| MCF_I2C_I2CR_IEN;		// Enable I2C module

	*(UINT32*)0x20000144 = (UINT32)I2cHandler;// Set interrupt vector (interrupt 17)

	MCF_INTC0_ICR17 = MCF_INTC_ICR_IL(1);			// Set interrupt level 1
	MCF_INTC0_IMRL &= ~MCF_INTC_IMRL_INT_MASK17;	// Reset interrupt controler mask bit
}

//
//	I2C interrupt handler
//
__declspec(interrupt) void I2cHandler(void)
{
	MCF_I2C_I2SR &= 0xfd;			// Cleat interrupt source
	if(MCF_I2C_I2CR & MCF_I2C_I2CR_MSTA)	// Master mode bit set?(Master only)
	{
		if(MCF_I2C_I2CR & MCF_I2C_I2CR_MTX)	// Tx mode?
		{	// Yes! It's TX mode
			if(I2cRepeat)		// Go repeat mode?
			{	// Yes! Switch to repeat start mode (receive)
				MCF_I2C_I2CR |= MCF_I2C_I2CR_RSTA;	// Set repeat start bit
				I2cResult = 0;		// Clear result code (redo from start)
				I2cRepeat = 0;		// Clear repeat flag (normal recieve sequence from now)
				I2cRecvMode = 1;	// Set receive mode flag	
				I2cSize = I2cRepeatSize;	// Set repeat data receive size
				I2cIndex = 0;			// Clear data store index for new receive sequence
				if(I2cSize == 1)	// Receive data size = 1?
					MCF_I2C_I2CR |= MCF_I2C_I2CR_TXAK;	// Set TXAK bit to send NAK
				else
					MCF_I2C_I2CR &= ~MCF_I2C_I2CR_TXAK;	// Clear TXAK bit to send ACK
				MCF_I2C_I2DR = I2cRepeatAdrs;		// Write recieve mode device address again
			}
			else if(I2cResult)	// Result already set?
			{	// Yes! Result is set then finish I2c session
				MCF_I2C_I2CR &= ~MCF_I2C_I2CR_MSTA;	// Clear master bit (will generate stop)
			}
			else
			{	// Transfer continue
				if(MCF_I2C_I2SR & MCF_I2C_I2SR_RXAK)	//Check ACK comes?
				{	// No ACK reply from device (must be no device in the I2c bus)
					MCF_I2C_I2CR &= ~MCF_I2C_I2CR_MSTA;	// Clear master bit(stop session)
					I2cResult = 2;	// Set error result code
				}
				else
				{	// ACK reply from I2c device
					if(I2cRecvMode)	// Receive mode?
					{
						I2cResult = 1;	// Set normal result code (ACK has come)
						MCF_I2C_I2CR &= ~MCF_I2C_I2CR_MTX;	// Clear TX bit and go receive mode
						(void)MCF_I2C_I2DR;	// Dummy read to recieve I2c device device data
					}
					else
					{	// Send mode
						if(I2cSize == 0) 	// No data to send (Address scan mode)
						{
							MCF_I2C_I2CR &= ~MCF_I2C_I2CR_MSTA;	// Clear master bit(stop session)
							I2cResult = 1;		// Set normal result code(1 is normal result)
							return;
						}
						else if(I2cIndex == I2cSize - 1) // Send data finish?
						{
							I2cResult = 1;		// Set normal result code(1 is normal result)
							if(I2cRepeatAdrs)	// Repeat flag set? 
								I2cRepeat = 1;	// Set repeat condition start flag
						}
						MCF_I2C_I2DR = I2cData[I2cIndex++];	// Send data to I2c bus
					}
				}
			}
		}
		else
		{	// RX mode
			if(I2cSize - I2cIndex == 2)		// Is it booby data?
				MCF_I2C_I2CR |= MCF_I2C_I2CR_TXAK;	// Ccleat TXAK bit to send NAK
			else if(I2cSize - I2cIndex == 1)	// Is it final data?
				MCF_I2C_I2CR &= ~MCF_I2C_I2CR_MSTA;	// Clear master bit to close I2c session
			I2cData[I2cIndex++] = MCF_I2C_I2DR;	// Store received data
		}
	}
}

//
//	Send I2C data
//
UINT32 I2cSend(UINT8 *data)
{
	I2cResult = 0;	// Clear result status 
	I2cRepeat = 0;	// Clear repeat conditopn flag
	I2cIndex = 0;		// Clear data index
	I2cRepeatAdrs = 0;	// Clear repeat address
  I2cSize = *data;// Set data send size
	I2cData = data + 2;	// Set data pointer
	I2cRecvMode = 0;// Clear recieve mode flag
	MCF_I2C_I2CR |= (MCF_I2C_I2CR_MTX | MCF_I2C_I2CR_MSTA);// Set MSTA & MTX bit same time
  MCF_I2C_I2DR = data[1] * 2;	// Send adrs (Send mode,RW bit = 0)
	while(!I2cResult || MCF_I2C_I2SR & MCF_I2C_I2SR_IBB)	// Wait session close
		SystemSleep();// Wait end of session
	return (I2cResult == 1);		// Check error happen
}

//
//	Get I2C data
//
UINT32 I2cGet(UINT8 *data)
{
	I2cResult = 0;		// Clear finish bit 
	I2cRepeat = 0;		// Clear repeat conditopn flag
	I2cIndex = 0;			// Clear send index
	I2cRepeatAdrs = 0;// Clear repeat address
  I2cSize = *data;// Set data send size
	I2cData = data + 2;// Set data pointer
	I2cRecvMode = 1;	// Set recieve mode flag
	MCF_I2C_I2CR |= (MCF_I2C_I2CR_MTX | MCF_I2C_I2CR_MSTA);// Set MSTA & MTX bit same time
	if(I2cSize == 1)		// Send data size = 1?
		MCF_I2C_I2CR |= MCF_I2C_I2CR_TXAK;	// Set TXAK bit from beggining(send NAK)
	else
		MCF_I2C_I2CR &= ~MCF_I2C_I2CR_TXAK;	// Clear TXAK bit first(send ACK)
  MCF_I2C_I2DR = data[1] * 2 + 1;	// Send adrs (Receive mode,RW bit = 1)
	while(!I2cResult || MCF_I2C_I2SR & MCF_I2C_I2SR_IBB)	// Wait session close
		SystemSleep();// Wait session finish
	return (I2cResult == 1);			// Check error happen
}

//
//	Repeat condition I2C data
//
UINT32 I2cRepeatGet(UINT8 *data)
{
	I2cResult = 0;		// Clear finish bit 
	I2cRepeat = 0;		// Set repeat mode flag
	I2cIndex = 0;			// Clear send index
	I2cRepeatAdrs = data[1] * 2 + 1;// Preset receive mode I2c address(RW = 1)
  I2cSize = *data;	// Set data send size
	I2cData = data + 2;	// Set data pointer
	I2cRepeatSize = *(data + *data + 2);	// Set repeat condition receive length
	I2cRecvMode = 0;	// Set recieve mode flag
	MCF_I2C_I2CR |= (MCF_I2C_I2CR_MTX | MCF_I2C_I2CR_MSTA);// Set MSTA & MTX bit same time
  MCF_I2C_I2DR = data[1] * 2;	// Send adrs (Send mode,RW bit = 0)
	while(!I2cResult || MCF_I2C_I2SR & MCF_I2C_I2SR_IBB)	// Wait session close
		SystemSleep();	// Wait session finish
	return (I2cResult == 1);	// Check error happen
}

//
//	Get char (space skip)
//
UINT8 GetChar()
{
	while(*p == ' ')	// Skip space
		p++;				// Advance pointer
	return (*p > 0x60)? *p - 0x20: *p;	// Return with char(to upper case)
}

//
//	Get hex one char (space skip)
//
UINT8 GetHexChar()
{
UINT8	data;	// Temp work
	data = GetChar();	// Get 1st char
	if(data >= '0' && data <='9')	// 0 - 9?
		return data - '0';	// Return 0 - 9
	if(data >= 'A' && data <= 'F')// A - F?
		return data - 'A' + 0x0a;	// Return A - F
	return 17;	// Return with non hex value
}

//
//	Get hex (space skip)
//
UINT16 GetHexByte()
{
UINT8	data, ch;	// Data work
	data = 0;	// Clear first
	ch = GetHexChar();	// Get 1st char
	if(ch == 17)	// Non hex char?
		return 256;	// return non byte value
	data = ch;		// Set higher 4 bit
	p++;	// Advance pointer
	ch = GetHexChar();	// Get 2nd char
	if(ch == 17)	// Non hex char?
	{
		p--;	// Back pointer as if it was 2 digit hex number
		return data;// Ignore 2nd char
	}
	data = (data << 4) | ch;	// Set lower 4 bit
	return data;	// Return with hex value
}

//
//	Make hex digit 8nibble data)
//
UINT8 MakeHexDigit(UINT8 data)
{
	return (data <= 9)? data +'0': data + 'A' - 10;// Make hex digit
}

//
//	Make hex digit (byte data)
//
UINT16 MakeHex2(UINT8 data)
{
	return (UINT16)((MakeHexDigit(data >> 4) << 8) | MakeHexDigit(data & 15));
}

//
//	Parse I2c control string
//
UINT32 I2cParser(UINT8 *data)
{
UINT8	*sendbuf;			// Send data buffer pointer
UINT8	*i2cbuf;			// I2c parameter table pointer
UINT8	sendlen;			// Send data length
UINT8 reclen;				// Receve data length
UINT8	adrs;					// I2c adrs
UINT16 temp;				// Temp value
UINT8	i;						// Loop work

	p = data;	// Set parse ptr
	adrs = 0;	// Clear I2c addres
	reclen = 0;	// Clear receive length
	sendlen = 0;// Clear send length
	temp = GetHexByte();	// Get address
	if(temp == 0 || temp == 256)	// No hex number?(Invalid adrs)
		return (UINT32)"?";	// Error return
	adrs = (UINT8)temp;	// Save I2c Adrs
	for(;;)		// Parse loop
	{
		switch(GetChar())
		{
		case 0:		// EOL ditected?
			if(adrs == 0)	// Address not set then ignore P command
				return (UINT32)"?";	// Error input line
			if(sendlen == 0 && reclen == 0)	// Both not set?
			{	// Address scan mode (0 byte send)
				i2cbuf = 0;		// Clear buffer for later MemoryFree
				sendbuf = (UINT8*)"";	// Set null result string first
				temp = adrs;	// Set I2c device adrs (RW = 0) & len = 0
				temp = (UINT16)I2cSend((UINT8*)&temp);	// Try to send 0 byte (adrs only)
			}
			else if(sendlen && reclen)	// Both length set?
			{	// Send & repeat receive mode
				temp = (sendlen > reclen)? sendlen: reclen;// Get buffer length
				i2cbuf = MemoryAlloc(temp + 3);	// Alloc buff to pas I2c driver
				i2cbuf[0] = sendlen;	// Sent sendlen
				i2cbuf[1] = adrs;	// Set I2c device adrs
				BufCopy(i2cbuf + 2, sendbuf, sendlen);	// Copy send data
				MemoryFree(sendbuf);	// Release send buffer
				i2cbuf[sendlen + 2] = reclen;	// Set repeat condition receive length
				temp = (UINT16)I2cRepeatGet(i2cbuf);	// Repeat condition write/read start
			}
			else if(sendlen)	// Send mode?
			{
				i2cbuf = MemoryAlloc(sendlen + 2);	// Alloc buff to pas I2c driver
				i2cbuf[0] = sendlen;	// Sent sendlen
				i2cbuf[1] = adrs;	// Set I2c device adrs
				BufCopy(i2cbuf + 2, sendbuf, sendlen);	// Copy send data
				MemoryFree(sendbuf);	// Release send buffer
				sendbuf = (UINT8*)"";	// Set null result string
				temp = (UINT16)I2cSend(i2cbuf);	// Send start
			}
			else if(reclen)	// Receive mode?
			{
				i2cbuf = MemoryAlloc(reclen + 3);	// Alloc buff to pas I2c driver
				i2cbuf[0] = reclen;	// Sent sendlen
				i2cbuf[1] = adrs;	// Set I2c device adrs
				temp = (UINT16)I2cGet(i2cbuf);	// Receive start
			}
			if(temp == 0)	// Some error happen?
				sendbuf = (UINT8*)"?";	// Set error string
			else if(reclen)	// if result exists?
			{
				sendbuf = MemoryAlloc(reclen * 2 + 1);	// Alloc result hex buffer
				for(i = 0; i < reclen; i++)	// Create result string
					*(UINT16*)&sendbuf[i * 2] = MakeHex2(i2cbuf[i + 2]);	// Set hex string
				sendbuf[i * 2] = '\0';	// Set EOS 
			}
			MemoryFree(i2cbuf);	// Release buffer to pass i2c driver
			return (UINT32)sendbuf;
			break;

		case 'R':		// Receive command
			if(adrs == 0 || reclen)	// Address not set or 2nd R command?
				break;	// Skip W command					
			p++;			// Advance pointer		
			temp = GetHexByte();	// Get receive length
			if(temp == 256)	// Invalid hex data?
				break;			// Ignore R command
			reclen = temp;// Set receive length
			break;

		case 'W':		// Write command
			if(adrs == 0 || sendlen)	// Address not set or 2nd W command?
				break;	// Skip W command
			p++;			// Advance pointer		
			sendbuf = p;	// Save current parse pointer
			for(;;sendlen++,p++)	// Data number count loop
			{
				temp = GetHexByte();// Check value
				if(temp == 256)	// No more hex data?
					break;	// Exit data count loop
			}
			if(sendlen)
			{
				p = sendbuf;	// Restore parse pointer
				sendbuf = MemoryAlloc(sendlen);	// Allocate send data buffer
				for(i = 0; i < sendlen; i++, p++)	// Data number count loop
					sendbuf[i] = GetHexByte();// Check value
			}
			p--;		// Back parse pointer to cancel next advance
			break;

		default:		// unknown data
			temp = GetHexByte();	// Dummy get hex number
			if(temp == 256)	// Invalid hex data?
				p--;		// Back parse pointer to cancel next advance
			break;
		}
		p++;		// Advance pointer		
	}
	return (UINT32)"?";	// Null result return
}


//
//	SystemSleep system call
//
asm void SystemSleep(void)
{
  move 0x210,a0		// Get SystemSleep internal address to a0
  jmp (a0)				// Jump SystemSleep library
}

//
//	Memory alloc system call
//
asm UINT8 *MemoryAlloc(UINT16 len)
{
  move 0x200,a0		// Get SystemSleep internal address to a0
  jmp (a0)				// Jump SystemSleep library
}

//
//	Memory free system call
//
asm void MemoryFree(UINT8 *ptr)
{
  move 0x20c,a0		// Get SystemSleep internal address to a0
  jmp (a0)				// Jump SystemSleep library
}

//
//	Buffer copy system call
//
asm void BufCopy(void *dst, void *src, UINT32 len)
{
  move 0x204,a0		// Get SystemSleep internal address to a0
  jmp (a0)				// Jump SystemSleep library
}

//
//	Putc system call
//
asm void TermPutc(UINT8 ch)
{
  move 0x300,a0		// Get SystemSleep internal address to a0
  jmp (a0)				// Jump SystemSleep library
}

