using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Net.Sockets;

namespace TempDisp
{
	public partial class Form1 : Form
	{
		bool online;					// Online flag
		TcpClient client;			// TCP/IP client object
		NetworkStream stream;	// Stream object

		//
		//	Form load constructor
		//
		public Form1()
		{
			InitializeComponent();// Init components
			online = false;				// Clear online flag
		}


		//
		//	Form load destructor
		//
		private void Form1_FormClosed(object sender, FormClosedEventArgs e)
		{
			if (online)				// TCP/IP session active?
				client.Close();	// Close session
		}

		//
		//	Get one line from ColdFire
		//
		private string getoneline()
		{
			int i, j;	// Read index
			byte[] recbuf = new byte[520];	// byte array to receive data
			i = 0;		// Clear read index
			for(;;)		// Repeat for one line
			{
				try
				{
					j = stream.Read(recbuf, i, 1);	// Try to read from ColdFire
				}
				catch
				{	// Error handler
					MessageBox.Show("TCP/IP^CAEg");	// show error
					return null;// return with NULL string
				}
				if (j == 0)		// 1 byte read successfully?
					break;			// No then buffer is empty
				if (recbuf[i] == 0x0a)	// LF has come?
					break;								// Exit read loop
				if (recbuf[i] == 0x0d)	// CR has come?
					continue;							// Ignore CR
				i++;										// Advance read index
			} 				
			return System.Text.Encoding.UTF8.GetString(recbuf, 0, i);	// Convert byte to string
		}

		//
		//	Put one line to ColdFire
		//
		private void putoneline(string text)
		{
			Byte[] data;				// byte array to send
			data = System.Text.Encoding.UTF8.GetBytes(text);	// Convert string
			stream.Write(data, 0, data.Length);		// Send byte array to TCP/IP stream
		}

		//
		//	Convert hex to temperture
		//
		private float settemp(string text)
		{
			int i, j;		// Loop counter & sum
			byte[] hex;	// byte array for hex digits
			hex = System.Text.Encoding.UTF8.GetBytes(text);	// Convert string to byte array
			for (i = 0, j = 0; i < 4; i++)				// Parse 4 digits
			{
				if (hex[i] >= '0' && hex[i] <= '9')	// 0 to 9?
					j = j * 16 + hex[i] - '0';				// Sum digit
				else if (hex[i] >= 'A' && hex[i] <= 'F')	// A to F?
					j = j * 16 + hex[i] - 'A' + 10;		// Sum digit
			}
			return (float)j / 128.0F;							// Divide by 128 to get temperture
		}

		//
		//	Wait SilentC prompt
		//
		private bool WaitPrompt(string pr)
		{
			string res;
			for (; ; )						// Skip OK response of SilentC
			{
				res = getoneline();	// get one line from ColdFire
				if (res == null)		// Some error happens?
					return true;			// Error retune (true)
				if (res.Contains(pr))	// Skip until "OK"
					break;						// Exit loop when OK has come
			}
			return false;					// Normal return
		}


		//
		//	Connect button click
		//
		private void button1_Click(object sender, EventArgs e)
		{
			try
			{
				client = new TcpClient(textBox1.Text, 23);	// Try to connect ColdFire telnet
			}
			catch
			{	// Error happen?
				MessageBox.Show("ColdFireɐڑł܂");// Show error message
				return;		// return (do nothing)
			}
			stream = client.GetStream();	// Get stream object for read/write
			stream.ReadTimeout = 3000;// Set timeout 3 seconds
			online = true;						// Set on line flag
			button1.Enabled = false;	// Disable connect button
			if(WaitPrompt("OK"))			// Wait OK prompt
				return;									// Return when some error happens
			putoneline("p::i\r");			// Send I2C initalize command
			WaitPrompt("OK");					// Wait OK prompt
		}

		//
		//	Display button proc
		//
		private void button2_Click(object sender, EventArgs e)
		{
		string res;						// Response from ColdFire
			if (online == false)				// Not connected yet?
				return;						// Do nothing
			putoneline("p::p(1)\r");		// Send I2C script command to ColdFire
			if (WaitPrompt("Ready"))		// Wait OK prompt
				return;						// Return when some error happens
			res = textBox2.Text + "r2\r";	// Create I2C address + read 2 bytes command 
			putoneline(res);		// Send I2C script command
			res = getoneline();	// Get echoback of I2C script
			res = getoneline();	// Get response from ColdFire
			if (res == null)		// Some error happens?
				label2.Text = "";	// Set blank string when error happens
			else
				label2.Text = settemp(res) + "";	// Create temperture string
			WaitPrompt("OK");		// Wait OK prompt
		}
	}
}